@@ -1,7 +1,7 @@
 /*****************************************************************************
  * VLC for iOS
  *****************************************************************************
- * Copyright (c) 2015, 2020 VideoLAN. All rights reserved.
+ * Copyright (c) 2015, 2020 - 2021 VideoLAN. All rights reserved.
  * $Id$
  *
  * Authors: Tobias Conradi <videolan # tobias-conradi.de>
@@ -22,9 +22,6 @@
 #import "VLCLocalNetworkServiceBrowserPlex.h"
 #import "VLCLocalNetworkServiceBrowserUPnP.h"
 #import "VLCLocalNetworkServiceBrowserNFS.h"
-#ifndef NDEBUG
-#import "VLCLocalNetworkServiceBrowserSAP.h"
-#endif
 #import "VLCLocalNetworkServiceBrowserDSM.h"
 #import "VLCLocalNetworkServiceBrowserBonjour.h"
 #import "VLCLocalNetworkServiceBrowserHTTP.h"
@@ -32,7 +29,6 @@
 #import "VLCNetworkServerLoginInformation+Keychain.h"
 
 #import "VLCRemoteBrowsingTVCell.h"
-#import "GRKArrayDiff+UICollectionView.h"
 
 #import "VLC-Swift.h"
 
@@ -98,12 +94,10 @@
                          [VLCLocalNetworkServiceBrowserPlex class],
                          [VLCLocalNetworkServiceBrowserNFS class],
                          [VLCLocalNetworkServiceBrowserBonjour class],
-#ifndef NDEBUG
-                         [VLCLocalNetworkServiceBrowserSAP class],
-#endif
                          ];
     self.discoveryController = [[VLCLocalServerDiscoveryController alloc] initWithServiceBrowserClasses:classes];
     self.discoveryController.delegate = self;
+    [self discoveryFoundSomethingNew];
 }
 
 - (NSString *)title {
@@ -350,6 +344,10 @@
     id<VLCNetworkServerBrowser> serverBrowser = nil;
     NSString *identifier = login.protocolIdentifier;
 
+    if (!login.address || login.address.length == 0) {
+        return;
+    }
+
     if ([identifier isEqualToString:VLCNetworkServerProtocolIdentifierFTP]) {
         serverBrowser = [VLCNetworkServerBrowserVLCMedia FTPNetworkServerBrowserWithLogin:login];
     } else if ([identifier isEqualToString:VLCNetworkServerProtocolIdentifierPlex]) {
@@ -371,12 +369,7 @@
 #pragma mark - VLCLocalServerDiscoveryController
 - (void)discoveryFoundSomethingNew
 {
-    NSString * (^mapServiceName)(id<VLCLocalNetworkService>) = ^NSString *(id<VLCLocalNetworkService> service) {
-        return [NSString stringWithFormat:@"%@: %@", service.serviceName, service.title];
-    };
-
     NSMutableArray<id<VLCLocalNetworkService>> *newNetworkServices = [NSMutableArray array];
-    NSMutableSet<NSString *> *addedNetworkServices = [[NSMutableSet alloc] init];
     VLCLocalServerDiscoveryController *discoveryController = self.discoveryController;
     NSUInteger sectionCount = [discoveryController numberOfSections];
     for (NSUInteger section = 0; section < sectionCount; ++section) {
@@ -385,26 +378,13 @@
             NSIndexPath *indexPath = [NSIndexPath indexPathForItem:index inSection:section];
             id<VLCLocalNetworkService> service = [discoveryController networkServiceForIndexPath:indexPath];
             if (service != nil) {
-                NSString *mappedName = mapServiceName(service);
-                if(![addedNetworkServices containsObject:mappedName]) {
-                    [addedNetworkServices addObject:mappedName];
-                    [newNetworkServices addObject:service];
-                }
+                [newNetworkServices addObject:service];
             }
         }
     }
 
-    NSArray *oldNetworkServices = self.networkServices;
-    GRKArrayDiff *diff = [[GRKArrayDiff alloc] initWithPreviousArray:oldNetworkServices
-                                                        currentArray:newNetworkServices
-                                                       identityBlock:mapServiceName
-                                                       modifiedBlock:nil];
-
-    [diff performBatchUpdatesWithCollectionView:self.collectionView
-                                        section:0
-                               dataSourceUpdate:^{
-                                   self.networkServices = newNetworkServices;
-                               } completion:nil];
+    self.networkServices = newNetworkServices;
+    [self.collectionView reloadData];
 
     _nothingFoundLabel.hidden = self.discoveryController.foundAnythingAtAll;
 }
