@@ -12,6 +12,7 @@
  *****************************************************************************/
 
 import Foundation
+import UIKit
 
 @objc protocol MediaCategoryViewControllerDelegate: NSObjectProtocol {
     func needsToUpdateNavigationbarIfNeeded(_ viewController: MediaCategoryViewController)
@@ -23,13 +24,14 @@ import Foundation
 class MediaCategoryViewController: UICollectionViewController, UISearchBarDelegate, IndicatorInfoProvider {
     // MARK: - Properties
     var model: MediaLibraryBaseModel
+    private var secondModel: MediaLibraryBaseModel
     private var services: Services
 
     var searchBar = UISearchBar(frame: .zero)
     var isSearching: Bool = false
     private let mediaGridCellNibIdentifier = "MediaGridCollectionCell"
     private var searchBarConstraint: NSLayoutConstraint?
-    private let searchDataSource: LibrarySearchDataSource
+    private var searchDataSource: LibrarySearchDataSource
     private let searchBarSize: CGFloat = 50.0
     private let userDefaults = UserDefaults.standard
     private var rendererButton: UIButton
@@ -57,22 +59,30 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
 
     @objc private lazy var sortActionSheet: ActionSheet = {
         var header: ActionSheetSortSectionHeader
-        var displayGridLayout: Bool = true
+        var isVideoModel: Bool = false
         var collectionModelName: String = ""
+        var secondSortModel: SortModel? = nil
 
-        if model is MediaGroupViewModel {
-            displayGridLayout = false
-        } else if let model = model as? CollectionModel {
-            if model.mediaCollection is VLCMLMediaGroup {
-                displayGridLayout = false
-            } else {
-                collectionModelName = String(describing: type(of: model.mediaCollection))
+        if let model = model as? CollectionModel {
+            if model.mediaCollection is VLCMLMediaGroup || model.mediaCollection is VideoModel {
+                isVideoModel = true
             }
+            collectionModelName = String(describing: type(of: model.mediaCollection)) + model.name
+        } else if let model = model as? MediaGroupViewModel {
+            isVideoModel = true
+            collectionModelName = model.name
+        } else if let model = model as? VideoModel {
+            isVideoModel = true
+            collectionModelName = secondModel.name
+            secondSortModel = model.sortModel
+        } else {
+            collectionModelName = model.name
         }
 
         header = ActionSheetSortSectionHeader(model: model.sortModel,
-                                              displayGridLayout: displayGridLayout,
-                                              currentModelType: collectionModelName + model.name)
+                                              secondModel: secondSortModel,
+                                              isVideoModel: isVideoModel,
+                                              currentModelType: collectionModelName)
 
         let actionSheet = ActionSheet(header: header)
         header.delegate = self
@@ -88,6 +98,7 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
                 UserDefaults.standard.set(sortingCriteria.rawValue, forKey: "\(kVLCSortDefault)\(model.name)")
             }
             self?.sortActionSheet.removeActionSheet()
+            self?.reloadData()
         }
         return actionSheet
     }()
@@ -100,6 +111,10 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         return setupEditBarButton()
     }()
 
+    private lazy var selectAllBarButton: UIBarButtonItem = {
+        return setupSelectAllButton()
+    }()
+
     private lazy var rendererBarButton: UIBarButtonItem = {
         return UIBarButtonItem(customView: rendererButton)
     }()
@@ -109,6 +124,11 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         let nib = Bundle.main.loadNibNamed(name, owner: self, options: nil)
         guard let emptyView = nib?.first as? VLCEmptyLibraryView else { fatalError("Can't find nib for \(name)") }
 
+        // Check if no playlists
+        if model is PlaylistModel {
+            emptyView.contentType = .noPlaylists
+        }
+
         // Check if it is inside a playlist
         if let collectionModel = model as? CollectionModel,
             collectionModel.mediaCollection is VLCMLPlaylist {
@@ -122,6 +142,9 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         return PresentationTheme.current.colors.statusBarStyle
     }
 
+    private var scrolledCellIndex: IndexPath = IndexPath()
+    private(set) var isAllSelected: Bool = false
+
     // MARK: - Initializers
 
     @available(*, unavailable)
@@ -131,16 +154,34 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
 
     init(services: Services, model: MediaLibraryBaseModel) {
         self.services = services
-        self.model = model
+
+        let videoModel = VideoModel(medialibrary: services.medialibraryService)
+        videoModel.secondName = model.name
+
+        if model is MediaGroupViewModel {
+            self.model = userDefaults.bool(forKey: kVLCSettingsDisableGrouping) ? videoModel : model
+            self.secondModel = userDefaults.bool(forKey: kVLCSettingsDisableGrouping) ? model : videoModel
+        } else {
+            self.model = model
+            self.secondModel = videoModel
+        }
+
         self.rendererButton = services.rendererDiscovererManager.setupRendererButton()
         self.searchDataSource = LibrarySearchDataSource(model: model)
 
         super.init(collectionViewLayout: UICollectionViewFlowLayout())
+
+        if PlaybackService.sharedInstance().renderer != nil {
+            rendererButton.isSelected = true
+        }
+
         let marqueeTitle = VLCMarqueeLabel()
         if let collection = model as? CollectionModel {
             title = collection.mediaCollection.title()
         }
         marqueeTitle.text = title
+        marqueeTitle.textColor = PresentationTheme.current.colors.navigationbarTextColor
+        marqueeTitle.font = UIFont.preferredCustomFont(forTextStyle: .headline).bolded
         self.navigationItem.titleView = marqueeTitle
         NotificationCenter.default.addObserver(self, selector: #selector(themeDidChange),
                                                name: .VLCThemeDidChangeNotification, object: nil)
@@ -151,6 +192,31 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         NotificationCenter.default.addObserver(self, selector: #selector(miniPlayerIsHidden),
                                                name: NSNotification.Name(rawValue: VLCPlayerDisplayControllerHideMiniPlayer),
                                                object: nil)
+        NotificationCenter.default.addObserver(self, selector: #selector(preferredContentSizeChanged(_:)),
+                                               name: UIContentSizeCategory.didChangeNotification,
+                                               object: nil)
+
+        if model is MediaGroupViewModel || model is VideoModel {
+            NotificationCenter.default.addObserver(self, selector: #selector(handleDisableGrouping),
+                                                   name: .VLCDisableGroupingDidChangeNotification,
+                                                   object: nil)
+        }
+
+
+    }
+
+    @objc private func handleDisableGrouping() {
+        let previousModel = model
+        model = secondModel
+        secondModel = previousModel
+        self.searchDataSource = LibrarySearchDataSource(model: model)
+        editController = EditController(mediaLibraryService: services.medialibraryService, model: model, presentingView: collectionView)
+        editController.delegate = self
+        model.sort(by: secondModel.sortModel.currentSort, desc: secondModel.sortModel.desc)
+        setupCollectionView()
+        cachedCellSize = .zero
+        collectionView?.collectionViewLayout.invalidateLayout()
+        reloadData()
     }
 
     @objc func miniPlayerIsShown() {
@@ -199,7 +265,7 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         // If we are a MediaGroupViewModel, check if there are no empty groups from ungrouping.
         if let mediaGroupModel = model as? MediaGroupViewModel {
             mediaGroupModel.files = mediaGroupModel.files.filter() {
-                return $0.nbMedia() != 0
+                return $0.nbTotalMedia() != 0
             }
         }
 
@@ -224,7 +290,11 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
     }
 
     @objc func reloadData() {
-        let timeInterval: Double = 0.3
+        // Timer set to 0.0 instead of 0.3 seconds because it causes a bug related to the swipe to delete.
+        // The timer was created two times due to several calls to reloadData().
+        // Meanwhile the user could try so swipe to delete and when the timer finally fired, the cell scrolled would be updated.
+        // Leading to another cell being scrolled instead of the first one.
+        let timeInterval: Double = 0.0
 
         if reloadTimer == nil {
             DispatchQueue.main.async {
@@ -247,6 +317,10 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         }
     }
 
+    func isEmptyCollectionView() -> Bool {
+        return collectionView?.numberOfItems(inSection: 0) == 0
+    }
+
     @available(*, unavailable)
     required init?(coder aDecoder: NSCoder) {
         fatalError("init(coder: ) has not been implemented")
@@ -256,7 +330,6 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         super.viewDidLoad()
         setupCollectionView()
         setupSearchBar()
-        _ = (MLMediaLibrary.sharedMediaLibrary() as! MLMediaLibrary).libraryDidAppear()
     }
 
     override func viewWillAppear(_ animated: Bool) {
@@ -279,6 +352,8 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         setupCollectionView() //Fixes crash that is caused due to layout change
         reloadData()
         showGuideOnLaunch()
+        setNavbarAppearance()
+        loadSort()
     }
 
     func loadSort() {
@@ -292,21 +367,21 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         self.model.sort(by: sortingCriteria, desc: desc)
     }
 
-    override func viewDidAppear(_ animated: Bool) {
-        super.viewDidAppear(animated)
-
-        loadSort()
+    private func setNavbarAppearance() {
+        if #available(iOS 13.0, *) {
+            navigationController?.navigationBar.standardAppearance = AppearanceManager.navigationbarAppearance()
+            navigationController?.navigationBar.scrollEdgeAppearance = AppearanceManager.navigationbarAppearance()
+        }
+        navigationController?.navigationBar.barTintColor = PresentationTheme.current.colors.navigationbarColor
+        setNeedsStatusBarAppearanceUpdate()
     }
 
     @objc func themeDidChange() {
         collectionView?.backgroundColor = PresentationTheme.current.colors.background
         searchBar.backgroundColor = PresentationTheme.current.colors.background
-
-        if #available(iOS 13.0, *) {
-            navigationController?.navigationBar.standardAppearance = AppearanceManager.navigationbarAppearance()
-            navigationController?.navigationBar.scrollEdgeAppearance = AppearanceManager.navigationbarAppearance()
+        if let marqueeLabel = navigationItem.titleView as? VLCMarqueeLabel {
+            marqueeLabel.textColor = PresentationTheme.current.colors.navigationbarTextColor
         }
-        setNeedsStatusBarAppearanceUpdate()
     }
 
     private func showGuideOnLaunch() {
@@ -330,6 +405,10 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
 
     // MARK: - Edit
 
+    override func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
+        resetScrollView()
+    }
+
     override func scrollViewDidScroll(_ scrollView: UIScrollView) {
         // This ensures that the search bar is always visible like a sticky while searching
         if isSearching {
@@ -361,6 +440,13 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         // Most of the edit logic is handled inside editController
         collectionView?.dataSource = editing ? editController : self
         collectionView?.delegate = editing ? editController : self
+        if #available(iOS 14.0, *) {
+            /// Those changes are highly recommended in order to prevent a UICollectionView gesture
+            /// issue when cells are embedding a UIScrollView
+            /// See https://code.videolan.org/umxprime/collection-view-bug
+            collectionView?.allowsSelectionDuringEditing = editing
+            collectionView?.allowsMultipleSelectionDuringEditing = editing
+        }
 
         editController.resetSelections(resetUI: true)
         displayEditToolbar()
@@ -369,6 +455,13 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
 
         searchBar.resignFirstResponder()
         searchBarConstraint?.constant = -self.searchBarSize
+
+        // When quitting the edit mode, reset all selection state
+        if isEditing == false {
+            isAllSelected = false
+            selectAllBarButton.image = UIImage(named: "emptySelectAll")
+        }
+
         reloadData()
     }
 
@@ -386,7 +479,10 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
     }
 
     func indicatorInfo(for pagerTabStripController: PagerTabStripViewController) -> IndicatorInfo {
-        let uiTestAccessibilityIdentifier = model is TrackModel ? VLCAccessibilityIdentifier.songs : nil
+        var uiTestAccessibilityIdentifier = model is TrackModel ? VLCAccessibilityIdentifier.songs : nil
+        if model is ArtistModel {
+            uiTestAccessibilityIdentifier = VLCAccessibilityIdentifier.artists
+        }
         return IndicatorInfo(title: model.indicatorName, accessibilityIdentifier: uiTestAccessibilityIdentifier)
     }
 }
@@ -404,10 +500,6 @@ private extension MediaCategoryViewController {
         }
     }
 
-    private func isEmptyCollectionView() -> Bool {
-        return collectionView?.numberOfItems(inSection: 0) == 0
-    }
-
     private func updateUIForContent() {
         if isSearching {
             return
@@ -419,6 +511,7 @@ private extension MediaCategoryViewController {
         }
         searchBar.isHidden = isEmpty || isEditing
         collectionView?.backgroundView = isEmpty ? emptyView : nil
+        updateBarButtonItems()
     }
 
     private func objects(from modelContent: VLCMLObject) -> [VLCMLObject] {
@@ -442,6 +535,11 @@ private extension MediaCategoryViewController {
         userActivity?.isEligibleForHandoff = true
         userActivity?.becomeCurrent()
     }
+
+    @objc func preferredContentSizeChanged(_ notification: Notification) {
+        cachedCellSize = .zero
+        collectionView?.collectionViewLayout.invalidateLayout()
+    }
 }
 
 // MARK: - NavigationItem
@@ -457,6 +555,16 @@ extension MediaCategoryViewController {
         return editButton
     }
 
+    private func setupSelectAllButton() -> UIBarButtonItem {
+        let selectAll = UIBarButtonItem(image: UIImage(named: "emptySelectAll"),
+                                        style: .plain, target: self,
+                                        action: #selector(handleSelectAll))
+        selectAll.accessibilityLabel = NSLocalizedString("BUTTON_SELECT_ALL", comment: "")
+        selectAll.accessibilityHint = NSLocalizedString("BUTTON_SELECT_ALL_HINT", comment: "")
+        return selectAll
+    }
+
+
     private func setupSortButton() -> UIButton {
         // Fetch sortButton configuration from MediaVC
         let sortButton = UIButton(frame: CGRect(x: 0, y: 0, width: 44, height: 44))
@@ -474,6 +582,13 @@ extension MediaCategoryViewController {
         return sortButton
     }
 
+    private func leftBarButtonItem() -> [UIBarButtonItem] {
+        var leftBarButtonItems = [UIBarButtonItem]()
+
+        leftBarButtonItems.append(selectAllBarButton)
+        return leftBarButtonItems
+    }
+
     private func rightBarButtonItems() -> [UIBarButtonItem] {
         var rightBarButtonItems = [UIBarButtonItem]()
 
@@ -538,6 +653,13 @@ extension MediaCategoryViewController {
         }
     }
 
+    @objc func handleSelectAll() {
+        isAllSelected = !isAllSelected
+        editController.selectAll()
+        selectAllBarButton.image = isAllSelected ? UIImage(named: "allSelected")
+            : UIImage(named: "emptySelectAll")
+    }
+
     @objc func handleSortShortcut() {
         model.sort(by: model.sortModel.currentSort, desc: !model.sortModel.desc)
     }
@@ -548,6 +670,7 @@ extension MediaCategoryViewController {
                                                                           target: self,
                                                                           action: #selector(handleEditing))]
             : rightBarButtonItems()
+        navigationItem.leftBarButtonItems = leftBarButtonItem()
         navigationItem.setHidesBackButton(isEditing, animated: true)
     }
 }
@@ -555,18 +678,24 @@ extension MediaCategoryViewController {
 // MARK: - VLCRendererDiscovererManagerDelegate
 
 extension MediaCategoryViewController: VLCRendererDiscovererManagerDelegate {
-    private func updateRightbarButtonItems() {
+    private func updateBarButtonItems() {
         if !isEditing {
             navigationItem.rightBarButtonItems = rightBarButtonItems()
+            navigationItem.leftBarButtonItem = nil
+        }
+
+        if isEmptyCollectionView() {
+            navigationItem.rightBarButtonItem = nil
+            navigationItem.leftBarButtonItem = nil
         }
     }
 
     @objc func addedRendererItem() {
-        updateRightbarButtonItems()
+        updateBarButtonItems()
     }
 
     @objc func removedRendererItem() {
-        updateRightbarButtonItems()
+        updateBarButtonItems()
     }
 }
 
@@ -608,6 +737,44 @@ extension MediaCategoryViewController {
 // MARK: - UICollectionViewDelegate - Private Helpers
 
 private extension MediaCategoryViewController {
+    private func generatePlayAction(for modelContent: VLCMLObject?, type: EditButtonType) {
+        if let media = modelContent as? VLCMLMedia {
+            let playbackController = PlaybackService.sharedInstance()
+            playbackController.mediaList.lock()
+            switch type {
+                case .play:
+                    playbackController.play(media)
+                case .playNextInQueue:
+                    playbackController.playMediaNextInQueue(media)
+                case .appendToQueue:
+                    playbackController.appendMediaToQueue(media)
+                default:
+                    assertionFailure("generatePlayAction: cannot be used with other actions")
+            }
+            playbackController.mediaList.unlock()
+        } else if let collection = modelContent as? MediaCollectionModel {
+            let playbackController = PlaybackService.sharedInstance()
+            playbackController.mediaList.lock()
+            let files: [VLCMLMedia]?
+            if collection is VLCMLAlbum {
+                files = collection.files(with: .trackNumber, desc: false)
+            } else {
+                files = collection.files(with: .default, desc: false)
+            }
+            switch type {
+                case .play:
+                    playbackController.playCollection(files)
+                case .playNextInQueue:
+                    playbackController.playCollectionNextInQueue(files)
+                case .appendToQueue:
+                    playbackController.appendCollectionToQueue(files)
+                default:
+                    assertionFailure("generatePlayAction: cannot be used with other actions")
+            }
+            playbackController.mediaList.unlock()
+        }
+    }
+
     @available(iOS 13.0, *)
     private func generateUIMenuForContent(at indexPath: IndexPath) -> UIMenu {
         let modelContentArray = isSearching ? searchDataSource.searchData : model.anyfiles
@@ -671,6 +838,21 @@ private extension MediaCategoryViewController {
                         }
                     }
                 })
+            case .play:
+                return $0.action({
+                    _ in
+                    self.generatePlayAction(for: modelContent, type: .play)
+                })
+            case .playNextInQueue:
+                return $0.action({
+                    _ in
+                    self.generatePlayAction(for: modelContent, type: .playNextInQueue)
+                })
+            case .appendToQueue:
+                return $0.action({
+                    _ in
+                    self.generatePlayAction(for: modelContent, type: .appendToQueue)
+                })
             }
         })
     }
@@ -684,9 +866,9 @@ extension MediaCategoryViewController {
         let modelContent = mediaObjectArray.objectAtIndex(index: indexPath.row)
 
         if let mediaGroup = modelContent as? VLCMLMediaGroup,
-            mediaGroup.nbMedia() == 1 && !mediaGroup.userInteracted() {
+            mediaGroup.nbTotalMedia() == 1 && !mediaGroup.userInteracted() {
             // We handle only mediagroups of video
-            guard let media = mediaGroup.media(of: .video)?.first else {
+            guard let media = mediaGroup.media(of: .unknown)?.first else {
                 assertionFailure("MediaCategoryViewController: Failed to fetch mediagroup video.")
                 return
             }
@@ -772,20 +954,32 @@ extension MediaCategoryViewController {
         }
 
         if let mediaGroup = mediaObject as? VLCMLMediaGroup {
-            guard let media = mediaGroup.media(of: .video)?.first else {
-                assertionFailure("MediaCategoryViewController: Failed to retrieve media")
+            guard let mediaArray = mediaGroup.media(of: .unknown) else {
+                assertionFailure("MediaCategoryViewController: Failed to retrieve media array")
                 return mediaCell
             }
-            services.medialibraryService.requestThumbnail(for: media)
+
+            // we show up to 4 thumbnails per group, so request those
+            for index in 0...3 {
+                if let media = mediaArray.objectAtIndex(index: index) {
+                    services.medialibraryService.requestThumbnail(for: media)
+                }
+            }
         } else if let media = mediaObject as? VLCMLMedia {
-            if media.type() == .video {
+            if media.type() == .unknown || media.type() == .video {
                 services.medialibraryService.requestThumbnail(for: media)
                 assert(media.mainFile() != nil, "The mainfile is nil")
             }
         }
 
+        if let mediaCell = mediaCell as? MediaCollectionViewCell {
+            mediaCell.delegate = self
+            mediaCell.isEditing = false
+        }
+
         mediaCell.media = mediaObject
         mediaCell.isAccessibilityElement = true
+
         return mediaCell
     }
 }
@@ -800,7 +994,9 @@ extension MediaCategoryViewController: UICollectionViewDelegateFlowLayout {
             //we don't have yet the updated safeare layout frame. This is addressed by relayouting from viewSafeAreaInsetsDidChange
             var toWidth = toSize.width != 0 ? toSize.width : collectionView.frame.size.width
             if #available(iOS 11.0, *) {
-                toWidth = collectionView.safeAreaLayoutGuide.layoutFrame.width
+                // In case of nested views, the safe area may not be updated.
+                // Getting its parent's safe area gives us the true updated safe area.
+                toWidth = parent?.view.safeAreaLayoutGuide.layoutFrame.width ?? collectionView.safeAreaLayoutGuide.layoutFrame.width
             }
             cachedCellSize = model.cellType.cellSizeForWidth(toWidth)
         }
@@ -880,23 +1076,33 @@ extension MediaCategoryViewController: ActionSheetSortSectionHeaderDelegate {
     }
 
     func actionSheetSortSectionHeader(_ header: ActionSheetSortSectionHeader, onSwitchIsOnChange: Bool, type: ActionSheetSortHeaderOptions) {
+        var prefix: String = ""
+        var suffix: String = ""
         if type == .descendingOrder {
             model.sort(by: model.sortModel.currentSort, desc: onSwitchIsOnChange)
-            userDefaults.set(onSwitchIsOnChange, forKey: "\(kVLCSortDescendingDefault)\(model.name)")
-        } else {
+            prefix = kVLCSortDescendingDefault
+            suffix = model is VideoModel ? secondModel.name : model.name
+        } else if type == .layoutChange {
             var collectionModelName: String = ""
+            var isVideoModel = false
             if let model = model as? CollectionModel {
+                if model.mediaCollection is VLCMLMediaGroup || model.mediaCollection is VideoModel {
+                    isVideoModel = true
+                }
                 collectionModelName = getTypeName(of: model.mediaCollection)
+            } else if model is VideoModel || model is MediaGroupViewModel {
+                isVideoModel = true
             }
 
-            userDefaults.set(onSwitchIsOnChange,
-                             forKey: "\(kVLCAudioLibraryGridLayout)\(collectionModelName + model.name)")
-
-            setupCollectionView()
-            cachedCellSize = .zero
-            collectionView?.collectionViewLayout.invalidateLayout()
-            reloadData()
+            prefix = isVideoModel ? kVLCVideoLibraryGridLayout : kVLCAudioLibraryGridLayout
+            suffix = collectionModelName + model.name
         }
+
+        userDefaults.set(onSwitchIsOnChange, forKey: "\(prefix)\(suffix)")
+        setupCollectionView()
+        cachedCellSize = .zero
+        collectionView?.collectionViewLayout.invalidateLayout()
+        reloadData()
     }
 }
 
@@ -915,13 +1121,13 @@ extension MediaCategoryViewController: EditControllerDelegate {
 
     func editControllerDidSelectMultipleItem(editContrller: EditController) {
         if let editToolbar = tabBarController?.editToolBar() {
-            editToolbar.enableMediaGroupButton(true)
+            editToolbar.enableEditActions(true)
         }
     }
 
     func editControllerDidDeSelectMultipleItem(editContrller: EditController) {
         if let editToolbar = tabBarController?.editToolBar() {
-            editToolbar.enableMediaGroupButton(false)
+            editToolbar.enableEditActions(false)
         }
     }
 
@@ -1015,10 +1221,10 @@ extension MediaCategoryViewController {
             var singleGroup = [VLCMLMediaGroup]()
             // Filter single groups
             singleGroup = mediaGroupModel.files.filter() {
-                return $0.nbMedia() == 1 && !$0.userInteracted()
+                return $0.nbTotalMedia() == 1 && !$0.userInteracted()
             }
             singleGroup.forEach() {
-                guard let media = $0.media(of: .video)?.first else {
+                guard let media = $0.media(of: .unknown)?.first else {
                     assertionFailure("MediaCategoryViewController: play: Failed to fetch media.")
                     return
                 }
@@ -1033,3 +1239,56 @@ extension MediaCategoryViewController {
         playbackController.playMedia(at: index, fromCollection: tracks)
     }
 }
+
+// MARK: - MediaCollectionViewCellDelegate
+
+extension MediaCategoryViewController: MediaCollectionViewCellDelegate {
+
+    func mediaCollectionViewCellHandleDelete(of cell: MediaCollectionViewCell) {
+        guard let indexPath = collectionView.indexPath(for: cell) else {
+            return
+        }
+
+        let modelContentArray = isSearching ? searchDataSource.searchData : model.anyfiles
+        let modelContent = modelContentArray.objectAtIndex(index: indexPath.row)
+        editController.editActions.objects = [modelContent!]
+        editController.editActions.delete()
+    }
+
+    func mediaCollectionViewCellMediaTapped(in cell: MediaCollectionViewCell) {
+        guard let indexPath = collectionView.indexPath(for: cell) else {
+            return
+        }
+
+        selectedItem(at: indexPath)
+    }
+
+    func mediaCollectionViewCellSetScrolledCellIndex(of cell: MediaCollectionViewCell?) {
+        if let cell = cell {
+            guard let indexPath = collectionView.indexPath(for: cell) else {
+                return
+            }
+
+            scrolledCellIndex = indexPath
+        }
+    }
+
+    func mediaCollectionViewCellGetScrolledCell() -> MediaCollectionViewCell? {
+        if scrolledCellIndex.isEmpty {
+            return nil
+        }
+
+        let cell = collectionView.cellForItem(at: scrolledCellIndex)
+        if let cell = cell as? MediaCollectionViewCell {
+            return cell
+        }
+
+        return nil
+    }
+
+    private func resetScrollView() {
+        if let mediaCell = mediaCollectionViewCellGetScrolledCell() {
+            mediaCell.resetScrollView()
+        }
+    }
+}
