@@ -2,7 +2,7 @@
  * VLCPlaybackService.m
  * VLC for iOS
  *****************************************************************************
- * Copyright (c) 2013-2018 VideoLAN. All rights reserved.
+ * Copyright (c) 2013-2022 VLC authors and VideoLAN
  * $Id$
  *
  * Authors: Felix Paul Kühne <fkuehne # videolan.org>
@@ -12,12 +12,12 @@
  *          Tobias Conradi <videolan # tobias-conradi.de>
  *          Sylver Bruneau <sylver.bruneau # gmail dot com>
  *          Winston Weinert <winston # ml1 dot net>
+ *          Maxime Chapelet <umxprime # videolabs.io>
  *
  * Refer to the COPYING file of the official project for license.
  *****************************************************************************/
 
 #import "VLCPlaybackService.h"
-#import "UIDevice+VLC.h"
 #import <AVFoundation/AVFoundation.h>
 #import "VLCRemoteControlService.h"
 #import "VLCMetadata.h"
@@ -33,7 +33,11 @@ NSString *const VLCPlaybackServicePlaybackMetadataDidChange = @"VLCPlaybackServi
 NSString *const VLCPlaybackServicePlaybackDidFail = @"VLCPlaybackServicePlaybackDidFail";
 NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackServicePlaybackPositionUpdated";
 
-@interface VLCPlaybackService () <VLCMediaPlayerDelegate, VLCMediaDelegate, VLCRemoteControlServiceDelegate>
+#if TARGET_OS_IOS
+@interface VLCPlaybackService () <VLCMediaPlayerDelegate, VLCMediaDelegate, VLCMediaListPlayerDelegate, VLCRemoteControlServiceDelegate, EqualizerViewDelegate>
+#else
+@interface VLCPlaybackService () <VLCMediaPlayerDelegate, VLCMediaDelegate, VLCMediaListPlayerDelegate, VLCRemoteControlServiceDelegate>
+#endif
 {
     VLCRemoteControlService *_remoteControlService;
     VLCMediaPlayer *_backgroundDummyPlayer;
@@ -55,6 +59,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
     BOOL _needsMetadataUpdate;
     BOOL _mediaWasJustStarted;
+    int _majorPositionChangeInProgress;
     BOOL _recheckForExistingThumbnail;
     BOOL _externalAudioPlaybackDeviceConnected;
 
@@ -66,6 +71,8 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     VLCDialogProvider *_dialogProvider;
     VLCCustomDialogRendererHandler *_customDialogHandler;
     VLCPlayerDisplayController *_playerDisplayController;
+
+    NSMutableArray *_openedLocalURLs;
 }
 
 @end
@@ -95,6 +102,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 {
     self = [super init];
     if (self) {
+        _fullscreenSessionRequested = YES;
         // listen to audiosessions and appkit callback
         _externalAudioPlaybackDeviceConnected = [self isExternalAudioPlaybackDeviceConnected];
         NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
@@ -128,6 +136,10 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
         // stay alive in background exclusively for Chromecast.
         _backgroundDummyPlayer = [[VLCMediaPlayer alloc] initWithOptions:@[@"--demux=rawaud"]];
         _backgroundDummyPlayer.media = [[VLCMedia alloc] initWithPath:@"/dev/zero"];
+
+        _mediaList = [[VLCMediaList alloc] init];
+
+        _openedLocalURLs = [[NSMutableArray alloc] init];
     }
     return self;
 }
@@ -161,6 +173,16 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
     _sessionWillRestart = _playerIsSetup;
     _playerIsSetup ? [self stopPlayback] : [self startPlayback];
+
+#if TARGET_OS_TV
+    VLCFullscreenMovieTVViewController *movieVC = [VLCFullscreenMovieTVViewController fullscreenMovieTVViewController];
+
+    if (![movieVC isBeingPresented]) {
+        [[UIApplication sharedApplication].keyWindow.rootViewController presentViewController:movieVC
+                                                                                     animated:YES
+                                                                                   completion:nil];
+    }
+#endif
 }
 
 - (VLCTime *)playedTime
@@ -197,23 +219,55 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     _actualVideoOutputView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
     _actualVideoOutputView.autoresizesSubviews = YES;
 
-    /* the chromecast-passthrough option cannot be set per media, so we need to set it per
+    /* the chromecast options cannot be set per media, so we need to set it per
      * media player instance however, potentially initialising an additional library instance
      * for this is costly, so this should be done only if needed */
-    BOOL chromecastPassthrough = [[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingCastingAudioPassthrough] boolValue];
+    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
+    BOOL chromecastPassthrough = [[userDefaults objectForKey:kVLCSettingCastingAudioPassthrough] boolValue];
+    int chromecastQuality = [[userDefaults objectForKey:kVLCSettingCastingConversionQuality] intValue];
+    NSMutableArray *libVLCOptions = [NSMutableArray array];
     if (chromecastPassthrough) {
-        _listPlayer = [[VLCMediaListPlayer alloc] initWithOptions:@[[@"--" stringByAppendingString:kVLCSettingCastingAudioPassthrough]]
+        [libVLCOptions addObject:[@"--" stringByAppendingString:kVLCSettingCastingAudioPassthrough]];
+    }
+    if (chromecastQuality != 2) {
+        [libVLCOptions addObject:[NSString stringWithFormat:@"--%@=%i", kVLCSettingCastingConversionQuality, chromecastQuality]];
+    }
+    if (libVLCOptions.count > 0) {
+        _listPlayer = [[VLCMediaListPlayer alloc] initWithOptions:libVLCOptions
                                                       andDrawable:_actualVideoOutputView];
     } else {
         _listPlayer = [[VLCMediaListPlayer alloc] initWithDrawable:_actualVideoOutputView];
     }
+    _listPlayer.delegate = self;
 
-    /* to enable debug logging for the playback library instance, switch the boolean below
-     * note that the library instance used for playback may not necessarily match the instance
-     * used for media discovery or thumbnailing */
-    _listPlayer.mediaPlayer.libraryInstance.debugLogging = NO;
+#if MEDIA_PLAYBACK_DEBUG
+    _listPlayer.mediaPlayer.libraryInstance.debugLogging = YES;
+    _listPlayer.mediaPlayer.libraryInstance.debugLoggingLevel = 4;
+#endif
+    BOOL saveDebugLogs = [userDefaults boolForKey:kVLCSaveDebugLogs];
+    if (saveDebugLogs) {
+        NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
+        NSString* logFilePath = [searchPaths[0] stringByAppendingPathComponent:@"Logs"];
+        NSFileManager *fileManager = [NSFileManager defaultManager];
+        ret = [fileManager fileExistsAtPath:logFilePath];
+        if (!ret) {
+            [fileManager createDirectoryAtPath:logFilePath withIntermediateDirectories:YES attributes:nil error:nil];
+        }
+        NSDate *date = [NSDate date];
+        NSDateFormatter *dateFormatter=[[NSDateFormatter alloc] init];
+        [dateFormatter setTimeZone:[NSTimeZone timeZoneWithName:@"UTC"]];
+        [dateFormatter setDateFormat:@"yyyy-MM-dd--HH-mm-ss"];
+        logFilePath = [logFilePath stringByAppendingPathComponent:[NSString stringWithFormat: @"vlcdebug-%@.log", [dateFormatter stringFromDate:date]]];
+        APLog(@"logging at '%@'", logFilePath);
+        [_listPlayer.mediaPlayer.libraryInstance setDebugLoggingToFile:logFilePath];
+    }
 
+    id<VLCFilter> newFilter = _listPlayer.mediaPlayer.adjustFilter;
+    [newFilter applyParametersFrom:_adjustFilter.mediaPlayerAdjustFilter];
+    newFilter.enabled = _adjustFilter.mediaPlayerAdjustFilter.isEnabled;
+    _adjustFilter = [[VLCPlaybackServiceAdjustFilter alloc] initWithMediaPlayerAdjustFilter:newFilter];
     _mediaPlayer = _listPlayer.mediaPlayer;
+
     [_mediaPlayer setDelegate:self];
     if ([[defaults objectForKey:kVLCSettingPlaybackSpeedDefaultValue] floatValue] != 0)
         [_mediaPlayer setRate: [[defaults objectForKey:kVLCSettingPlaybackSpeedDefaultValue] floatValue]];
@@ -330,8 +384,17 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
         _mediaPlayer = nil;
         _listPlayer = nil;
     }
+
+    for (NSURL *url in _openedLocalURLs) {
+        [url stopAccessingSecurityScopedResource];
+        NSLog(@"%@", url);
+    }
+    _openedLocalURLs = nil;
+    _openedLocalURLs = [[NSMutableArray alloc] init];
+
     if (!_sessionWillRestart) {
         _mediaList = nil;
+        _mediaList = [[VLCMediaList alloc] init];
     }
     _playerIsSetup = NO;
     [_shuffleStack removeAllObjects];
@@ -379,6 +442,17 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if ([self.delegate respondsToSelector:@selector(playbackPositionUpdated:)])
         [self.delegate playbackPositionUpdated:self];
 
+    if (_majorPositionChangeInProgress >= 1) {
+        [self.metadata updateExposedTimingFromMediaPlayer:_listPlayer.mediaPlayer];
+        _majorPositionChangeInProgress++;
+
+        /* we wait up to 10 time change intervals for the major position change
+         * to take effect, afterwards we give up, safe battery and let the OS calculate the position */
+        if (_majorPositionChangeInProgress == 10) {
+            _majorPositionChangeInProgress = 0;
+        }
+    }
+
     [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackPositionUpdated
                                                         object:self];
 }
@@ -459,6 +533,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 - (void)setPlaybackPosition:(float)position
 {
     _mediaPlayer.position = position;
+    _majorPositionChangeInProgress = 1;
 }
 
 - (void)setSubtitleDelay:(float)subtitleDeleay
@@ -471,65 +546,6 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     return _mediaPlayer.currentVideoSubTitleDelay/1000.;
 }
 
-- (float)hue
-{
-    return _mediaPlayer.hue;
-}
-
-- (void)setHue:(float)hue
-{
-    _mediaPlayer.hue = hue;
-}
-
-- (float)contrast
-{
-    return _mediaPlayer.contrast;
-}
-
-- (void)setContrast:(float)contrast
-{
-    _mediaPlayer.contrast = contrast;
-}
-
-- (float)brightness
-{
-    return _mediaPlayer.brightness;
-}
-
-- (void)setBrightness:(float)brightness
-{
-    _mediaPlayer.brightness = brightness;
-}
-
-- (float)saturation
-{
-    return _mediaPlayer.saturation;
-}
-
-- (void)setSaturation:(float)saturation
-{
-    _mediaPlayer.saturation = saturation;
-}
-
-- (void)setGamma:(float)gamma
-{
-    _mediaPlayer.gamma = gamma;
-}
-
-- (float)gamma
-{
-    return _mediaPlayer.gamma;
-}
-
-- (void)resetFilters
-{
-    _mediaPlayer.hue = 0.;
-    _mediaPlayer.contrast = 1.;
-    _mediaPlayer.brightness = 1.;
-    _mediaPlayer.saturation = 1.;
-    _mediaPlayer.gamma = 1.;
-}
-
 - (void)toggleRepeatMode
 {
     if (_listPlayer.repeatMode == VLCRepeatAllItems) {
@@ -561,7 +577,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
 - (NSInteger)numberOfAudioTracks
 {
-    return _mediaPlayer.audioTrackIndexes.count;
+    return [_mediaPlayer numberOfAudioTracks];
 }
 
 - (NSInteger)numberOfVideoSubtitlesIndexes
@@ -587,21 +603,21 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     } else if (index == count) {
         return NSLocalizedString(@"DOWNLOAD_SUBS_FROM_OSO", nil);
     }
-    return nil;
+    return @"";
 }
 
 - (NSString *)audioTrackNameAtIndex:(NSInteger)index
 {
     if (index >= 0 && index < _mediaPlayer.audioTrackNames.count)
         return _mediaPlayer.audioTrackNames[index];
-    return nil;
+    return @"";
 }
 
 - (NSDictionary *)titleDescriptionsDictAtIndex:(NSInteger)index
 {
     if (index >= 0 && index < _mediaPlayer.titleDescriptions.count)
         return _mediaPlayer.titleDescriptions[index];
-    return nil;
+    return [NSDictionary dictionary];
 }
 
 - (NSDictionary *)chapterDescriptionsDictAtIndex:(NSInteger)index
@@ -609,7 +625,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     NSArray *chapterDescriptions = [_mediaPlayer chapterDescriptionsOfTitle:_mediaPlayer.currentTitleIndex];
     if (index >= 0 && index < chapterDescriptions.count)
         return chapterDescriptions[index];
-    return nil;
+    return [NSDictionary dictionary];
 }
 
 - (void)selectAudioTrackAtIndex:(NSInteger)index
@@ -749,6 +765,15 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackDidPause object:self];
 }
 
+- (void)playItemAtIndex:(NSUInteger)index
+{
+    VLCMedia *media = [_mediaList mediaAtIndex:index];
+    [_listPlayer playItemAtNumber:[NSNumber numberWithUnsignedInteger:index]];
+    _mediaPlayer.media = media;
+    if ([self.delegate respondsToSelector:@selector(prepareForMediaPlayback:)])
+        [self.delegate prepareForMediaPlayback:self];
+}
+
 - (void)setShuffleMode:(BOOL)shuffleMode
 {
     _shuffleMode = shuffleMode;
@@ -805,12 +830,18 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
     if (nextIndex < 0) {
         if (self.repeatMode == VLCRepeatAllItems) {
+#if TARGET_OS_IOS
+            [_delegate savePlaybackState:self];
+#endif
             [_listPlayer next];
             [[NSNotificationCenter defaultCenter]
              postNotificationName:VLCPlaybackServicePlaybackMetadataDidChange object:self];
         }
         return NO;
     }
+#if TARGET_OS_IOS
+    [_delegate savePlaybackState:self];
+#endif
 
     [_listPlayer playItemAtNumber:@(nextIndex)];
     [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackMetadataDidChange object:self];
@@ -820,17 +851,23 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 - (BOOL)previous
 {
     if (_mediaList.count > 1) {
-        [_listPlayer previous];
-        [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackMetadataDidChange object:self];
-    }
-    else {
+        VLCTime *playedTime = self.playedTime;
+        if (playedTime.value.longLongValue / 2000 >= 1) {
+            self.playbackPosition = .0;
+        } else {
+#if TARGET_OS_IOS
+            [_delegate savePlaybackState:self];
+#endif
+            [_listPlayer previous];
+            [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackMetadataDidChange object:self];
+        }
+    } else {
         NSNumber *skipLength = [[NSUserDefaults standardUserDefaults] valueForKey:kVLCSettingPlaybackBackwardSkipLength];
         [_mediaPlayer jumpBackward:skipLength.intValue];
     }
     return YES;
 }
 
-
 - (void)jumpForward:(int)interval
 {
     [_mediaPlayer jumpForward:interval];
@@ -1075,6 +1112,16 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     return [_mediaPlayer preAmplification];
 }
 
+- (unsigned int)numberOfBands
+{
+    return [_mediaPlayer numberOfBands];
+}
+
+- (CGFloat)frequencyOfBandAtIndex:(unsigned int)index
+{
+    return [_mediaPlayer frequencyOfBandAtIndex:index];
+}
+
 #pragma mark - AVAudioSession Notification Observers
 
 - (void)handleInterruption:(NSNotification *)notification
@@ -1343,6 +1390,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 {
     float positionDiff = playbackTime - [self.metadata.elapsedPlaybackTime floatValue];
     [_mediaPlayer jumpForward:positionDiff];
+    _majorPositionChangeInProgress = 1;
 }
 
 #pragma mark - helpers
@@ -1354,7 +1402,8 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
               kVLCSettingStretchAudio : [[defaults objectForKey:kVLCSettingStretchAudio] boolValue] ? kVLCSettingStretchAudioOnValue : kVLCSettingStretchAudioOffValue,
               kVLCSettingTextEncoding : [defaults objectForKey:kVLCSettingTextEncoding],
               kVLCSettingSkipLoopFilter : [defaults objectForKey:kVLCSettingSkipLoopFilter],
-              kVLCSettingHardwareDecoding : [defaults objectForKey:kVLCSettingHardwareDecoding]
+              kVLCSettingHardwareDecoding : [defaults objectForKey:kVLCSettingHardwareDecoding],
+              kVLCSettingNetworkRTSPTCP : [defaults objectForKey:kVLCSettingNetworkRTSPTCP]
     };
 }
 
@@ -1379,4 +1428,13 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     [_playerDisplayController dismissPlaybackView];
 }
 
+#pragma mark - VLCMediaListPlayerDelegate
+
+- (void)mediaListPlayer:(VLCMediaListPlayer *)player nextMedia:(VLCMedia *)media
+{
+    if ([_delegate respondsToSelector:@selector(playbackService:nextMedia:)]) {
+        [_delegate playbackService:self nextMedia:media];
+    }
+}
+
 @end
